package jp.co.yahoo.hadoop.ml.common;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.MapWritable;
import org.apache.hadoop.io.NullWritable;
import org.apache.hadoop.io.SequenceFile;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.io.VIntWritable;

import net.reduls.gomoku.Morpheme;
import net.reduls.gomoku.Tagger;

/**
 * Common Utility Class
 */
public class CommonUtil {
	public static final String DOC_COUNT_PER_CAT_KEY = "DocCountPerCat";
	public static final String BIAS_KEY = "$$$bias$$$";
	public static final String allowedWordClass = "名詞|形容詞|動詞";
	public static final double SMOOTHING = 0.1;
	public static final double MAX_AGE = 100;

	private CommonUtil() {
		throw new AssertionError();
	}

	/**
	 * Check if the word is content word.
	 * 
	 * @param feature
	 *            word class string
	 * @return true if content word
	 */
	public static boolean isContentWord(String feature) {
		String wordClass = feature.split(",")[0];
		return wordClass.matches("(" + allowedWordClass + ")");
	}

	/**
	 * Read a test file and extract features.
	 * 
	 * @param input
	 *            file path
	 * @return the feature vector of the test file
	 * @throws IOException
	 */
	public static MapWritable readTestFile(String input) throws IOException {
		MapWritable testMap = new MapWritable();
		FileReader in = new FileReader(input);
		BufferedReader br = new BufferedReader(in);
		try {
			String line;
			while ((line = br.readLine()) != null) {
				for (Morpheme m : Tagger.parse(line)) {
					Text word = new Text(m.surface);
					if (!isContentWord(m.feature))
						continue;
					VIntWritable freq = (VIntWritable) (testMap.get(word));
					if (freq == null) {
						testMap.put(word, new VIntWritable(1));
					} else {
						freq.set(freq.get() + 1);
					}
				}
			}
		} finally {
			br.close();
			in.close();
		}
		return testMap;
	}

	/**
	 * Read a weight vector from a file for binary classification or regression.
	 * 
	 * @param input
	 *            file path
	 * @return weight vector
	 * @throws IOException
	 */
	public static MapWritable readWeightFile(String input) throws IOException {
		MapWritable weightMap = new MapWritable();

		File trainFile = new File(input);
		Path trainPath = new Path(trainFile.toURI().toString());

		Configuration conf = new Configuration();
		FileSystem fs = trainPath.getFileSystem(conf);

		SequenceFile.Reader reader = new SequenceFile.Reader(fs, trainPath,
				conf);
		NullWritable key = NullWritable.get();
		reader.next(key, weightMap);
		reader.close();
		/*
		 * for (Entry<Writable, Writable> entry : weightMap.entrySet()) { Text
		 * word = (Text) entry.getKey(); double x = ((DoubleWritable)
		 * entry.getValue()).get(); System.out.println(word + ":" + x); }
		 */
		return weightMap;
	}

	/**
	 * Read a weight vector from a file for multi-class classification.
	 * 
	 * @param input
	 *            file path
	 * @return weight vectors
	 * @throws IOException
	 */
	public static Map<VIntWritable, MapWritable> readMulticlassWeightFile(
			String input) throws IOException {
		Map<VIntWritable, MapWritable> weightMap = new HashMap<VIntWritable, MapWritable>();

		File trainFile = new File(input);
		Path trainPath = new Path(trainFile.toURI().toString());

		Configuration conf = new Configuration();
		FileSystem fs = trainPath.getFileSystem(conf);

		SequenceFile.Reader reader = new SequenceFile.Reader(fs, trainPath,
				conf);
		try {
			VIntWritable key = new VIntWritable();
			MapWritable value = new MapWritable();
			while (reader.next(key, value)) {
				weightMap.put(new VIntWritable(key.get()), new MapWritable(
						value));
			}
		} finally {
			reader.close();
		}

		return weightMap;
	}
}